@@ -1,4 +1,15 @@
-from keras_cv_attention_models.mlp_family.mlp_mixer import MLPMixer, MLPMixerS32, MLPMixerS16, MLPMixerB32, MLPMixerB16, MLPMixerL32, MLPMixerL16, MLPMixerH14, mlp_block, mixer_block
+from keras_cv_attention_models.mlp_family.mlp_mixer import (
+    MLPMixer,
+    MLPMixerS32,
+    MLPMixerS16,
+    MLPMixerB32,
+    MLPMixerB16,
+    MLPMixerL32,
+    MLPMixerL16,
+    MLPMixerH14,
+    mlp_block,
+    mlp_mixer_block,
+)
 from keras_cv_attention_models.mlp_family.res_mlp import ResMLP, ResMLP12, ResMLP24, ResMLP36, ResMLP_B24, ChannelAffine
 from keras_cv_attention_models.mlp_family.gated_mlp import GMLP, GMLPTiny16, GMLPS16, GMLPB16, spatial_gating_block
 from keras_cv_attention_models.mlp_family.wave_mlp import WaveMLP, WaveMLP_T, WaveMLP_S, WaveMLP_M, WaveMLP_B, phase_aware_token_mixing
@@ -122,7 +133,7 @@ Trainable params: 693
 Non-trainable params: 0
 _________________________________________________________________
 """
-mixer_block.__doc__ = __mlp_mixer_head_doc__ + """
+mlp_mixer_block.__doc__ = __mlp_mixer_head_doc__ + """
 MLP Mixer block
 
 Args:
@@ -136,7 +147,7 @@ Examples:
 
 >>> from keras_cv_attention_models import attention_layers
 >>> inputs = keras.layers.Input([14 * 14, 9])
->>> nn = attention_layers.mixer_block(inputs, tokens_mlp_dim=14 * 14, channels_mlp_dim=9 * 4)
+>>> nn = attention_layers.mlp_mixer_block(inputs, tokens_mlp_dim=14 * 14, channels_mlp_dim=9 * 4)
 >>> print(f"{nn.shape = }")
 nn.shape = TensorShape([None, 196, 9])
 
