@@ -1,6 +1,9 @@
-import tensorflow as tf
-from tensorflow import keras
+import math
+from keras_cv_attention_models import backend
+from keras_cv_attention_models.backend import layers, models, functional, image_data_format
 from keras_cv_attention_models.attention_layers import (
+    activation_by_name,
+    add_with_layer_scale_and_drop_block,
     batchnorm_with_activation,
     conv2d_no_bias,
     ChannelAffine,
@@ -42,7 +45,8 @@ PRETRAINED_DICT = {
 
 
 def bottle_in_linear_out_block(inputs, out_channel, strides=1, expand_ratio=4, use_shortcut=False, drop_rate=0, activation="swish", name=""):
-    hidden_dim = int(inputs.shape[-1] * expand_ratio)
+    input_channel = inputs.shape[-1 if image_data_format() == "channels_last" else 1]
+    hidden_dim = int(input_channel * expand_ratio)
     deep = conv2d_no_bias(inputs, hidden_dim, kernel_size=1, strides=1, name=name + "deep_1_")
     deep = batchnorm_with_activation(deep, activation=activation, name=name + "deep_1_")
     deep = depthwise_conv2d_no_bias(deep, kernel_size=3, strides=strides, padding="SAME", name=name + "deep_2_")
@@ -51,22 +55,23 @@ def bottle_in_linear_out_block(inputs, out_channel, strides=1, expand_ratio=4, u
     deep = batchnorm_with_activation(deep, activation=None, name=name + "deep_3_")
     deep = drop_block(deep, drop_rate=drop_rate, name=name + "deep_")
 
-    out = keras.layers.Add()([inputs, deep]) if use_shortcut else deep
+    out = layers.Add()([inputs, deep]) if use_shortcut else deep
     return out
 
 
 def linear_self_attention(inputs, qkv_bias=False, out_bias=False, attn_axis=2, attn_dropout=0, name=None):
-    input_channel = inputs.shape[-1]
+    channel_axis = -1 if image_data_format() == "channels_last" else 1
+    input_channel = inputs.shape[channel_axis]
     qkv = conv2d_no_bias(inputs, 1 + input_channel * 2, kernel_size=1, use_bias=qkv_bias, name=name and name + "qkv_")
-    query, key, value = tf.split(qkv, [1, input_channel, input_channel], axis=-1)
-    context_score = keras.layers.Softmax(axis=attn_axis, name=name and name + "attention_scores")(query)  # on patch_hh * patch_ww dimension
-    context_score = keras.layers.Dropout(attn_dropout, name=name and name + "attn_drop")(context_score) if attn_dropout > 0 else context_score
+    query, key, value = functional.split(qkv, [1, input_channel, input_channel], axis=channel_axis)
+    context_score = layers.Softmax(axis=attn_axis, name=name and name + "attention_scores")(query)  # on patch_hh * patch_ww dimension
+    context_score = layers.Dropout(attn_dropout, name=name and name + "attn_drop")(context_score) if attn_dropout > 0 else context_score
     # print(f"{query.shape = }, {key.shape = }, {value.shape = }, {context_score.shape = }")
 
-    context_vector = keras.layers.Multiply()([key, context_score])  # [batch, height, width, input_channel]
-    context_vector = tf.reduce_sum(context_vector, keepdims=True, axis=attn_axis)  # on patch_hh * patch_ww dimension
+    context_vector = layers.Multiply()([key, context_score])  # [batch, height, width, input_channel]
+    context_vector = functional.reduce_sum(context_vector, keepdims=True, axis=attn_axis)  # on patch_hh * patch_ww dimension
 
-    out = tf.nn.relu(value) * context_vector
+    out = functional.relu(value) * context_vector
     out = conv2d_no_bias(out, input_channel, kernel_size=1, use_bias=out_bias, name=name and name + "output")
     return out
 
@@ -78,8 +83,30 @@ def mhsa_mlp_block(
     qkv_bias=True,
     mlp_ratio=4,
     num_norm_groups=-1,  # -1 or 0 for V1 using layer_norm, or 1 for V2 using group_norm
-    use_linear_attention=False,  # False for V1, True for V2
-    use_conv_mlp=False,  # False for V1, True for V2
+    mlp_drop_rate=0,
+    attn_drop_rate=0,
+    drop_rate=0,
+    layer_scale=-1,
+    activation="gelu",
+    name=None,
+):
+    # print(f"{inputs.shape = }")
+    attn = group_norm(inputs, groups=num_norm_groups, axis=-1, name=name + "attn_") if num_norm_groups > 0 else layer_norm(inputs, axis=-1, name=name + "attn_")
+    attn = multi_head_self_attention(attn, num_heads, qkv_bias=qkv_bias, out_bias=True, attn_dropout=attn_drop_rate, name=name and name + "attn_mhsa_")
+    attn = add_with_layer_scale_and_drop_block(inputs, attn, layer_scale=layer_scale, drop_rate=drop_rate, axis=-1, name=name and name + "1_")
+
+    mlp = group_norm(attn, groups=num_norm_groups, axis=-1, name=name + "mlp_") if num_norm_groups > 0 else layer_norm(attn, axis=-1, name=name + "mlp_")
+    mlp = mlp_block(mlp, int(out_channel * mlp_ratio), drop_rate=mlp_drop_rate, activation=activation, name=name and name + "mlp_")
+    return add_with_layer_scale_and_drop_block(attn, mlp, layer_scale=layer_scale, drop_rate=drop_rate, axis=-1, name=name and name + "2_")
+
+
+def linear_mhsa_mlp_block(
+    inputs,
+    out_channel,
+    num_heads=4,
+    qkv_bias=True,
+    mlp_ratio=4,
+    num_norm_groups=-1,  # -1 or 0 for V1 using layer_norm, or 1 for V2 using group_norm
     mlp_drop_rate=0,
     attn_drop_rate=0,
     drop_rate=0,
@@ -88,85 +115,96 @@ def mhsa_mlp_block(
     name=None,
 ):
     attn = group_norm(inputs, groups=num_norm_groups, name=name + "attn_") if num_norm_groups > 0 else layer_norm(inputs, name=name + "attn_")
-    if use_linear_attention:  # V2
-        if num_norm_groups > 0:
-            attn = keras.layers.Reshape(attn.shape[1:])(attn)  # Or will throw error when converting tflite, if GroupNorm is followed by Conv2D
-        attn = linear_self_attention(attn, qkv_bias=qkv_bias, out_bias=True, attn_dropout=attn_drop_rate, name=name and name + "attn_mhsa_")
-    else:  # V1
-        attn = multi_head_self_attention(attn, num_heads, qkv_bias=qkv_bias, out_bias=True, attn_dropout=attn_drop_rate, name=name and name + "attn_mhsa_")
-    attn = ChannelAffine(use_bias=False, weight_init_value=layer_scale, name=name and name + "1_gamma")(attn) if layer_scale >= 0 else attn
-    attn = drop_block(attn, drop_rate=drop_rate, name=name and name + "attn_")
-    attn_out = keras.layers.Add(name=name and name + "attn_out")([inputs, attn])
+    attn = layers.Reshape(attn.shape[1:])(attn)  # Or will throw error when converting tflite, if GroupNorm is followed by Conv2D
+    attn = linear_self_attention(attn, qkv_bias=qkv_bias, out_bias=True, attn_dropout=attn_drop_rate, name=name and name + "attn_mhsa_")
+    attn = add_with_layer_scale_and_drop_block(inputs, attn, layer_scale=layer_scale, drop_rate=drop_rate, name=name and name + "1_")
 
-    mlp = group_norm(attn_out, groups=num_norm_groups, name=name + "mlp_") if num_norm_groups > 0 else layer_norm(attn_out, name=name + "mlp_")
-    if use_conv_mlp and num_norm_groups > 0:  # V2
-        mlp = keras.layers.Reshape(mlp.shape[1:])(mlp)  # Or will throw error when converting tflite, if GroupNorm is followed by Conv2D
-    mlp = mlp_block(mlp, int(out_channel * mlp_ratio), drop_rate=mlp_drop_rate, use_conv=use_conv_mlp, activation=activation, name=name and name + "mlp_")
-    mlp = ChannelAffine(use_bias=False, weight_init_value=layer_scale, name=name and name + "2_gamma")(mlp) if layer_scale >= 0 else mlp
-    mlp = drop_block(mlp, drop_rate=drop_rate, name=name and name + "mlp_")
-    return keras.layers.Add(name=name and name + "out")([attn_out, mlp])
+    mlp = group_norm(attn, groups=num_norm_groups, name=name + "mlp_") if num_norm_groups > 0 else layer_norm(attn, name=name + "mlp_")
+    mlp = layers.Reshape(mlp.shape[1:])(mlp)  # Or will throw error when converting tflite, if GroupNorm is followed by Conv2D
+    mlp = mlp_block(mlp, int(out_channel * mlp_ratio), drop_rate=mlp_drop_rate, use_conv=True, activation=activation, name=name and name + "mlp_")
+    return add_with_layer_scale_and_drop_block(attn, mlp, layer_scale=layer_scale, drop_rate=drop_rate, name=name and name + "2_")
 
 
 def transformer_pre_process(inputs, out_channel, patch_size=2, resize_first=False, use_depthwise=False, patches_to_batch=True, activation="swish", name=""):
+    height_axis, width_axis, channel_axis = (1, 2, 3) if image_data_format() == "channels_last" else (2, 3, 1)
     nn = inputs
 
     if resize_first:  # V2
-        patch_hh, patch_ww = int(tf.math.ceil(nn.shape[1] / patch_size)), int(tf.math.ceil(nn.shape[2] / patch_size))
+        patch_hh, patch_ww = int(math.ceil(nn.shape[height_axis] / patch_size)), int(math.ceil(nn.shape[width_axis] / patch_size))
         # print(f"transformer_pre_process before resize: {nn.shape = }")
-        if patch_hh * patch_size != nn.shape[1] or patch_ww * patch_size != nn.shape[2]:
-            nn = tf.image.resize(nn, [patch_hh * patch_size, patch_ww * patch_size], method="bilinear")
+        if patch_hh * patch_size != nn.shape[height_axis] or patch_ww * patch_size != nn.shape[width_axis]:
+            nn = functional.resize(nn, [patch_hh * patch_size, patch_ww * patch_size], method="bilinear")
 
     if use_depthwise:  # V2
         nn = depthwise_conv2d_no_bias(nn, kernel_size=3, strides=1, padding="SAME", name=name + "pre_1_")
     else:  # V1
-        nn = conv2d_no_bias(nn, nn.shape[-1], kernel_size=3, strides=1, padding="SAME", name=name + "pre_1_")
+        nn = conv2d_no_bias(nn, nn.shape[channel_axis], kernel_size=3, strides=1, padding="SAME", name=name + "pre_1_")
     nn = batchnorm_with_activation(nn, activation=activation, name=name + "pre_1_")
     nn = conv2d_no_bias(nn, out_channel, kernel_size=1, strides=1, name=name + "pre_2_")
 
     if not resize_first:  # V1
-        patch_hh, patch_ww = int(tf.math.ceil(nn.shape[1] / patch_size)), int(tf.math.ceil(nn.shape[2] / patch_size))
+        patch_hh, patch_ww = int(math.ceil(nn.shape[height_axis] / patch_size)), int(math.ceil(nn.shape[width_axis] / patch_size))
         # print(f"transformer_pre_process before resize: {nn.shape = }")
-        if patch_hh * patch_size != nn.shape[1] or patch_ww * patch_size != nn.shape[2]:
-            nn = tf.image.resize(nn, [patch_hh * patch_size, patch_ww * patch_size], method="bilinear")
+        if patch_hh * patch_size != nn.shape[height_axis] or patch_ww * patch_size != nn.shape[width_axis]:
+            nn = functional.resize(nn, [patch_hh * patch_size, patch_ww * patch_size], method="bilinear")
 
     # Extract patchs, limit transpose permute length <= 4
     # [batch, height, width, channel] -> [batch, height // 2, 2, width // 2, 2, channel] -> [batch * 4, height // 2, width // 2, channel]
     # print(f"transformer_pre_process after resize: {nn.shape = }")
-    nn = tf.reshape(nn, [-1, patch_ww, patch_size, out_channel])  # [batch * patch_hh * h_patch_size, patch_ww, w_patch_size, channel]
-    nn = tf.transpose(nn, [0, 2, 1, 3])  # [batch * patch_hh * h_patch_size, w_patch_size, patch_ww, channel]
-    nn = tf.reshape(nn, [-1, patch_hh, patch_size * patch_size, patch_ww * out_channel])  # [batch, patch_hh, h_patch_size * w_patch_size, patch_ww * channel]
-    nn = tf.transpose(nn, [0, 2, 1, 3])  # [batch, h_patch_size * w_patch_size, patch_hh, patch_ww * channel]
-    extract_shape = [-1, patch_hh, patch_ww, out_channel] if patches_to_batch else [-1, patch_size * patch_size, patch_hh * patch_ww, out_channel]
-    nn = tf.reshape(nn, extract_shape)
-
+    if image_data_format() == "channels_last":
+        nn = functional.reshape(nn, [-1, patch_ww, patch_size, out_channel])  # [B * patch_hh * h_patch_size, patch_ww, w_patch_size, C]
+        nn = functional.transpose(nn, [0, 2, 1, 3])  # [B * patch_hh * h_patch_size, w_patch_size, patch_ww, C]
+        nn = functional.reshape(nn, [-1, patch_hh, patch_size * patch_size, patch_ww * out_channel])  # [B, patch_hh, h_patch_size * w_patch_size, patch_ww * C]
+        nn = functional.transpose(nn, [0, 2, 1, 3])  # [B, h_patch_size * w_patch_size, patch_hh, patch_ww * C]
+        extract_shape = [-1, patch_hh, patch_ww, out_channel] if patches_to_batch else [-1, patch_size * patch_size, patch_hh * patch_ww, out_channel]
+        nn = functional.reshape(nn, extract_shape)  # channels_last
+    else:  # [B, C, patch_hh * h_patch_size, patch_ww * w_patch_size]
+        nn = functional.reshape(nn, [-1, patch_size, patch_ww, patch_size])  # [B * C * patch_hh, h_patch_size, patch_ww, w_patch_size]
+        nn = functional.transpose(nn, [0, 2, 1, 3])  # [B * C * patch_hh, patch_ww, h_patch_size, w_patch_size]
+        nn = functional.reshape(nn, [-1, out_channel, patch_hh * patch_ww, patch_size * patch_size])  # [B, C, patch_hh * patch_ww, h_patch_size * w_patch_size]
+        if patches_to_batch:  # V1 -> channels_last, V2 keep channels_first, `patch_hh * patch_ww` -> attn_axis=2
+            nn = functional.transpose(nn, [0, 3, 2, 1])  # [B, h_patch_size * w_patch_size, patch_hh * patch_ww, C]
+            nn = functional.reshape(nn, [-1, patch_hh, patch_ww, out_channel])  # channels_last
     return nn
 
 
 def transformer_post_process(inputs, pre_attn, out_channel, patch_size=2, patch_height=-1, activation="swish", name=""):
-    if patch_height == -1:  # V1, [batch * 4, height // 2, width // 2, channel]
-        patch_hh, patch_ww, channel = inputs.shape[1], inputs.shape[2], inputs.shape[-1]
-    else:  # V2, [batch, 4, height // 2 * width // 2, channel]
-        patch_hh, patch_ww, channel = patch_height, inputs.shape[2] // patch_height, inputs.shape[-1]
+    height_axis, width_axis, channel_axis = (1, 2, 3) if image_data_format() == "channels_last" else (2, 3, 1)
+    patches_to_batch = patch_height == -1
+    if patches_to_batch:  # V1, [batch * 4, height // 2, width // 2, channel], channels_last
+        patch_hh, patch_ww, channel = inputs.shape[1], inputs.shape[2], inputs.shape[3]
+    else:  # V2, [batch, 4, height // 2 * width // 2, channel], channels_last or channels_first
+        patch_hh, patch_ww, channel = patch_height, inputs.shape[2] // patch_height, inputs.shape[channel_axis]
     # print(f"{patch_hh = }, {patch_ww = }, {channel = }, {inputs.shape = }")
 
-    # [batch * 4, height // 2, width // 2, channel] -> [batch, height // 2, 2, width // 2, 2, channel] -> [batch, height, width, channel]
-    nn = tf.reshape(inputs, [-1, patch_size * patch_size, patch_hh, patch_ww * channel])  # [batch, h_patch_size * w_patch_size, patch_hh, patch_ww * channel]
-    nn = tf.transpose(nn, [0, 2, 1, 3])  # [batch, patch_hh, h_patch_size * w_patch_size, patch_ww * channel]
-    nn = tf.reshape(nn, [-1, patch_size, patch_ww, channel])  # [batch * patch_hh * h_patch_size, w_patch_size, patch_ww, channel]
-    nn = tf.transpose(nn, [0, 2, 1, 3])  # [batch * patch_hh * h_patch_size, patch_ww, w_patch_size, channel]
-    nn = tf.reshape(nn, [-1, patch_hh * patch_size, patch_ww * patch_size, channel])
+    # [B * 4, height // 2, width // 2, C] -> [B, height // 2, 2, width // 2, 2, C] -> [B, height, width, C]
+    if image_data_format() == "channels_last":
+        nn = functional.reshape(inputs, [-1, patch_size * patch_size, patch_hh, patch_ww * channel])  # [B, h_patch_size * w_patch_size, patch_hh, patch_ww * C]
+        nn = functional.transpose(nn, [0, 2, 1, 3])  # [B, patch_hh, h_patch_size * w_patch_size, patch_ww * C]
+        nn = functional.reshape(nn, [-1, patch_size, patch_ww, channel])  # [B * patch_hh * h_patch_size, w_patch_size, patch_ww, C]
+        nn = functional.transpose(nn, [0, 2, 1, 3])  # [B * patch_hh * h_patch_size, patch_ww, w_patch_size, C]
+        nn = functional.reshape(nn, [-1, patch_hh * patch_size, patch_ww * patch_size, channel])
+    else:
+        nn = inputs
+        if patches_to_batch:
+            nn = functional.reshape(nn, [-1, patch_size * patch_size, patch_hh * patch_ww, channel])  # [B, h_patch_size * w_patch_size, patch_hh * patch_ww, C]
+            nn = functional.transpose(nn, [0, 3, 2, 1])  # [B, C, patch_hh * patch_ww, h_patch_size * w_patch_size]
+        nn = functional.reshape(nn, [-1, patch_ww, patch_size, patch_size])  # [B * C * patch_hh, patch_ww, h_patch_size, w_patch_size]
+        nn = functional.transpose(nn, [0, 2, 1, 3])  # [B * C * patch_hh, h_patch_size, patch_ww, w_patch_size]
+        nn = functional.reshape(nn, [-1, channel, patch_hh * patch_size, patch_ww * patch_size])
+
     # print(f"transformer_post_process before resize: {nn.shape = }")
-    if pre_attn is not None and (nn.shape[1] != pre_attn.shape[1] or nn.shape[2] != pre_attn.shape[2]):
-        nn = tf.image.resize(nn, [pre_attn.shape[1], pre_attn.shape[2]], method="bilinear")
+    if pre_attn is not None and (nn.shape[height_axis] != pre_attn.shape[height_axis] or nn.shape[width_axis] != pre_attn.shape[width_axis]):
+        nn = functional.resize(nn, [pre_attn.shape[height_axis], pre_attn.shape[width_axis]], method="bilinear")
     # print(f"transformer_post_process after resize: {nn.shape = }")
 
     nn = conv2d_no_bias(nn, out_channel, kernel_size=1, strides=1, name=name + "post_1_")
     nn = batchnorm_with_activation(nn, activation=activation, name=name + "post_1_")
     if pre_attn is not None:  # V1
-        nn = tf.concat([pre_attn, nn], axis=-1)
+        nn = functional.concat([pre_attn, nn], axis=channel_axis)
         nn = conv2d_no_bias(nn, out_channel, kernel_size=3, strides=1, padding="SAME", name=name + "post_2_")
         nn = batchnorm_with_activation(nn, activation=activation, name=name + "post_2_")
-    return keras.layers.Activation("linear", name=name + "output")(nn)  # Identity, Just need a name here
+    return layers.Activation("linear", name=name + "output")(nn)  # Identity, Just need a name here
 
 
 def MobileViT(
@@ -178,13 +216,11 @@ def MobileViT(
     expand_ratio=4,
     stem_width=16,
     patch_size=2,
-    patches_to_batch=True,  # True for V1, False for V2
     resize_first=False,  # False for V1, True for V2
     use_depthwise=False,  # False for V1, True for V2
     use_fusion=True,  # True for V1, False for V2
     num_norm_groups=-1,  # -1 or 0 for V1 using layer_norm, or 1 for V2 using group_norm
     use_linear_attention=False,  # False for V1, True for V2
-    use_conv_mlp=False,  # False for V1, True for V2
     output_num_features=640,
     layer_scale=-1,
     input_shape=(256, 256, 3),
@@ -197,9 +233,13 @@ def MobileViT(
     model_name="mobilevit",
     kwargs=None,
 ):
-    inputs = keras.layers.Input(input_shape)
+    # Regard input_shape as force using original shape if first element is None or -1,
+    # else assume channel dimention is the one with min value in input_shape, and put it first or last regarding image_data_format
+    input_shape = backend.align_input_shape_by_image_data_format(input_shape)
+    inputs = layers.Input(input_shape)
     nn = conv2d_no_bias(inputs, stem_width, kernel_size=3, strides=2, padding="same", name="stem_")
     nn = batchnorm_with_activation(nn, activation=activation, name="stem_")
+    height_axis, width_axis, channel_axis = (1, 2, 3) if image_data_format() == "channels_last" else (2, 3, 1)
 
     # Save line width
     mhsa_mlp_block_common_kwargs = {
@@ -207,8 +247,6 @@ def MobileViT(
         "qkv_bias": True,
         "mlp_ratio": 2,
         "num_norm_groups": num_norm_groups,
-        "use_linear_attention": use_linear_attention,
-        "use_conv_mlp": use_conv_mlp,
         "activation": activation,
     }
 
@@ -223,7 +261,7 @@ def MobileViT(
         for block_id in range(num_block):
             name = stack_name + "block{}_".format(block_id + 1)
             stride = stride if block_id == 0 else 1
-            use_shortcut = False if stride != 1 or nn.shape[-1] != out_channel else True
+            use_shortcut = False if stride != 1 or nn.shape[channel_axis] != out_channel else True
             block_drop_rate = drop_connect_rate * global_block_id / total_blocks
             global_block_id += 1
             if is_conv_block or block_id == 0:  # First transformer block is also a conv block .
@@ -231,15 +269,25 @@ def MobileViT(
             else:
                 if block_id == 1:  # pre
                     pre_attn = nn if use_fusion else None
-                    patch_height = -1 if patches_to_batch else int(tf.math.ceil(nn.shape[1] / patch_size))
+                    patches_to_batch = not use_linear_attention
+                    patch_height = -1 if patches_to_batch else int(math.ceil(nn.shape[height_axis] / patch_size))
                     nn = transformer_pre_process(nn, attn_channel, patch_size, resize_first, use_depthwise, patches_to_batch, activation=activation, name=name)
-                nn = mhsa_mlp_block(nn, attn_channel, layer_scale=layer_scale, **mhsa_mlp_block_common_kwargs, name=name)
+
+                if use_linear_attention:  # channels_last for Tensorflow, channels_first for PyTorch
+                    nn = linear_mhsa_mlp_block(nn, attn_channel, layer_scale=layer_scale, **mhsa_mlp_block_common_kwargs, name=name)
+                else:  # channels_last for both Tensorflow or PyTorch
+                    nn = mhsa_mlp_block(nn, attn_channel, layer_scale=layer_scale, **mhsa_mlp_block_common_kwargs, name=name)
+
                 if block_id == num_block - 1:  # post
-                    nn = group_norm(nn, groups=num_norm_groups, name=name + "post_") if num_norm_groups > 0 else layer_norm(nn, name=name + "post_")
+                    norm_axis = "auto" if use_linear_attention else -1
+                    if use_linear_attention:
+                        nn = group_norm(nn, groups=num_norm_groups, axis=norm_axis, name=name + "post_")
+                    else:
+                        nn = layer_norm(nn, axis=norm_axis, name=name + "post_")
                     nn = transformer_post_process(nn, pre_attn, out_channel, patch_size, patch_height, activation=post_activation, name=name)
 
     nn = output_block(nn, output_num_features, activation, num_classes, drop_rate=dropout, classifier_activation=classifier_activation)
-    model = keras.models.Model(inputs, nn, name=model_name)
+    model = models.Model(inputs, nn, name=model_name)
     add_pre_post_process(model, rescale_mode="raw01")
     reload_model_weights(model, PRETRAINED_DICT, "mobilevit", pretrained)
     return model
